#!/usr/bin/python

import subprocess
import sched, time
from phabletutils import settings
from phabletutils import environment
from phabletutils import ubuntuimage

def execute(command):
    subprocess.call(command, shell=True)

def execute_output(command):
    return subprocess.check_output(command.split())

def is_device_plugged():
    return execute_output("adb get-state").rstrip() == "device"


def flash():
    if not is_device_plugged():
        print("No device connected.")
        return

    version = get_device_image_version()
    if version == get_latest_image_version():
        print "Device is up to date at version %s" % version
        return

    # install the latest green image: on host run...
    execute("phablet-flash ubuntu-system --channel devel --bootstrap")

    # disable the intro
    execute("phablet-config edges-intro --disable")

    # make image RW and reboot; only do this for testing debs, if only testing
    # click packages this can be skipped.
    execute("phablet-config writable-image")

    # For click apps, this will download the tests for all preinstalled apps
    execute("phablet-click-test-setup")

    # Enable autopilot to reach the confined click apps, running this can take 
    # a few to several minutes depending on the amount of click apps installed
    execute("phablet-config autopilot --dbus-probe enable")


def get_latest_image_version():
    device = environment.detect_device("")
    json = ubuntuimage.get_json_from_index(device, "devel", 0, settings.system_image_uri)
    return json["version"]

def get_device_image_version():
    output = execute_output("adb shell cat /etc/ubuntu-build")
    return int(output.rstrip())

def run_every(seconds, function):
    def wrapper():
        function()
        s.enter(seconds, 1, wrapper, ())

    s = sched.scheduler(time.time, time.sleep)
    s.enter(0, 1, wrapper, ())
    s.run()

def main():
    run_every(3600, flash)

if __name__ == '__main__':
    main()
