#!/usr/bin/python

import subprocess
import json
import os, os.path
import glob
import urllib, zipfile
import tempfile
import sys
import signal

def execute(command):
    subprocess.call(command, shell=True)

def execute_output(command):
    return subprocess.check_output(command.split())

def execute_background(command, shell=False):
    return subprocess.Popen(command if shell else command.split() , shell=shell)


def deb_uri_from_merge_request(merge_request_uri):
    # TODO
    return merge_request_uri

def unzip(zipfilename, destination):
    zfile = zipfile.ZipFile(zipfilename)
    zfile.extractall(destination)

def download_debs(merge_request_uri):
    deb_uri = deb_uri_from_merge_request(merge_request_uri)
    tmp_directory = tempfile.mkdtemp()
    target_zip = os.path.join(tmp_directory, "packages.zip")

    print("Downloading packages from %s" % deb_uri)
    urllib.urlretrieve(deb_uri, target_zip)
    unzip(target_zip, tmp_directory)

    deb_directory = os.path.join(tmp_directory, "output")
    return deb_directory

def is_package_installed(package):
    return bool(execute_output("adb shell dpkg --status %s 2> /dev/null | grep Status" % package))

def install_deb_on_device(path_to_deb):
    execute("adb push %s /tmp/" % path_to_deb)
    execute("adb shell dpkg -i --status-fd=0 /tmp/%s 0>&1 >/dev/null | grep \"\ installed\"" % os.path.basename(path_to_deb))

def install_debs_from_merge_request(merge_request_uri):
    packages = []
    deb_directory = download_debs(merge_request_uri)
    debs = glob.glob(os.path.join(deb_directory, "*.deb"))
    for deb in debs:
        package = os.path.basename(deb).split('_')[0]
        if is_package_installed(package):
            print("Installing %s..." % package)
            packages.append(package)
            install_deb_on_device(deb)

    return packages

def run_tests(application, logfile):
    print("Running autopilot tests for %s" % application)
    command = "phablet-test-run -x \"cd autopilot; autopilot run -v %s >/dev/null 2>> %s\""
    execute(command % (application, logfile))

def list_applications():
    apps = [
        "ubuntuuitoolkit",
#"ubuntuuitoolkit.tests.test_emulators.ActionSelectionPopoverTestCase.test_click_unexisting_button",
        "calendar_app",
        "music_app",
        "notes_app",
        "stock_ticker",
        "sudoku_app",
        "ubuntu_calculator_app",
        "ubuntu_clock_app",
        "ubuntu_filemanager_app",
        "ubuntu_rssreader_app",
        "ubuntu_terminal_app",
        "ubuntu_weather_app"
    ]
    return apps

def run_all_tests(logfile):
    display_process = execute_background("adb shell powerd-cli display on bright")
    # empty logfile
    execute("adb shell \"echo \"\" > %s\"" % logfile)
    grep = "grep -v 'WARNING testcase\|INFO _launcher\|INFO logging\|INFO run\|DEBUG\|\*\*\*\*\*\|gcc'"     
    log_process = execute_background("adb shell \"tail -f -c +0 %s | %s\"" % (logfile, grep), shell=True)
    for application in list_applications():
        run_tests(application, logfile)
    display_process.kill()
    log_process.kill()


def reinstall_from_archive(packages):
    packages_string = ' '.join(packages)
    redirects = "0>&1 2>/dev/null >/dev/null"
    grep = "grep \"not-installed\""
    execute("adb shell dpkg --remove --force-all --status-fd=0 %s %s | %s" % (packages_string, redirects, grep))
    print("Reinstalling %s from the Ubuntu archive..." % packages_string)
    execute("adb shell apt-get install --reinstall %s >/dev/null" % packages_string)



def main():
#    mr = "https://code.launchpad.net/~fboucault/ubuntu-ui-toolkit/visual_compact_widgets/+merge/204792"
#    debs = "http://jenkins.qa.ubuntu.com/job/ubuntu-ui-toolkit-trusty-armhf-ci/652/artifact/work/output/*zip*/output.zip"
    debs = sys.argv[1]
    logfile = sys.argv[2]
    packages = install_debs_from_merge_request(debs)

    def cleanup(signal, frame):
        reinstall_from_archive(packages)
        sys.exit(0)

    signal.signal(signal.SIGTERM, cleanup)
 
    run_all_tests(logfile)
    reinstall_from_archive(packages)

if __name__ == '__main__':
    main()

